#include <string.h>
#include <stdio.h>
#include <jcode.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/X.h>
#include <X11/keysym.h>
#include <X11/Xlocale.h>
#include "TPager.h"
#include "getstr.h"

TScrlbar(Display *display, struct PAGER *Pager, struct SCROLL *Scroll, XEvent *event)
{
	static int		ButtonPressFlg = 0;
	static int		ButtonPressCount;
	static int		MOVEBOX = 0;
	static int		ywin;
	static int		tmp;

	if (MOVEBOX == 1){
		MOVEBOX = MOVE_BOX(display, Pager, Scroll, ywin, event);
	}

	if (event->type == ButtonRelease){
		ButtonPressFlg = 0;
		return;
	}

	if (event->type == EnterNotify){
		if (event->xcrossing.window == Scroll->win_up1)
			XSetWindowBorder(display, Scroll->win_up1, Scroll->ScrlFG); 
		if (event->xcrossing.window == Scroll->win_up2)
			XSetWindowBorder(display, Scroll->win_up2, Scroll->ScrlFG);
		if (event->xcrossing.window == Scroll->win_down1)
			XSetWindowBorder(display, Scroll->win_down1, Scroll->ScrlFG);
		if (event->xcrossing.window == Scroll->win_down2)
			XSetWindowBorder(display, Scroll->win_down2, Scroll->ScrlFG);
		return;
	}

	if (event->type == LeaveNotify){
		if (event->xcrossing.window == Scroll->win_up1)
			XSetWindowBorder(display, Scroll->win_up1, Scroll->ScrlBG); 
		if (event->xcrossing.window == Scroll->win_up2)
			XSetWindowBorder(display, Scroll->win_up2, Scroll->ScrlBG);
		if (event->xcrossing.window == Scroll->win_down1)
			XSetWindowBorder(display, Scroll->win_down1, Scroll->ScrlBG);
		if (event->xcrossing.window == Scroll->win_down2)
			XSetWindowBorder(display, Scroll->win_down2, Scroll->ScrlBG);
		return;
	}

	if (event->type == ClientMessage){
		if (event->xclient.message_type == XInternAtom(display, "Scroll-DrawScrollBar", False)){
			DrawScrollBar(display, Scroll, (double)event->xclient.data.l[0]/10000.0);
			return;
		}
	}

	if (event->type == ButtonPress){
		if (event->xany.window == Scroll->win_up1){
			SendScrl(display, Pager, -1);
			return;
		}

		if (event->xany.window == Scroll->win_down1){
			SendScrl(display, Pager, 1);
			return;
		}

		if (event->xany.window == Scroll->win_down2){
			SendHalfPage(display, Pager, 1);
			return;
		}

		if (event->xany.window == Scroll->win_up2){
			SendHalfPage(display, Pager, -1);
			return;
		}

		if (event->xany.window == Scroll->scrbar_down){
			SendPage(display, Pager, 1);
			return;
		}
		if (event->xany.window == Scroll->scrbar_up){
			SendPage(display, Pager, -1);
			return;
		}
		if (event->xany.window == Scroll->scrbar_box){
			MOVEBOX = 1;
			XGetGeometry(display, Scroll->scrbar_box, &tmp, &tmp, &ywin, &tmp, &tmp, &tmp, &tmp);
			ywin = ywin - event->xbutton.y_root, event;
			return;
		}
	}
}


DrawScrollBar(Display *display, struct SCROLL *Scroll, double percent)
{
	int		sbpos;

	sbpos = percent / 100.0 * (Scroll->ScrlHeit - Scroll->ScrlBoxHeit) + 1;

    XMoveWindow(display, Scroll->scrbar_box, 0, sbpos);
    XResizeWindow(display, Scroll->scrbar_up, Scroll->ScrlBoxWid, sbpos);
    XMoveResizeWindow(display, Scroll->scrbar_down, 0, sbpos + Scroll->ScrlBoxHeit, Scroll->ScrlBoxWid, Scroll->ScrlHeit - (sbpos + Scroll->ScrlBoxHeit)+2);
}

MOVE_BOX(Display *display, struct PAGER *Pager, struct SCROLL *Scroll, int y00, XEvent *event)
{
    int             y;
    double          dy, dlines;

	if (event->xany.window != Scroll->scrbar_box) return 1;

    if (Pager->lines<Pager->LINES){
        Bell(display, Pager->window, Pager->BELL);
        return 0;
    }

	if ( event->type == ButtonRelease && event->xbutton.button == 1){
		SendChangeLine(display, Pager, Pager->pos);
		return 0;
	}

	if ( event->type == MotionNotify && event->xbutton.button == 1){
		y = event->xmotion.y_root + y00;
		if ( y < 1 )                y = 0;
		if ( y > Scroll->ScrlHeit - Scroll->ScrlBoxHeit ) y = Scroll->ScrlHeit - Scroll->ScrlBoxHeit;
		XMoveWindow(display, Scroll->scrbar_box, 0, y+1);
		dy = y; dlines = Pager->lines;

		if (Pager->pos != (int) (dlines-Pager->LINES)*(dy)/(Scroll->ScrlHeit - Scroll->ScrlBoxHeit)){
			Pager->pos = (int) (dlines-Pager->LINES)*(dy)/(Scroll->ScrlHeit - Scroll->ScrlBoxHeit);
			if (XPending(display) == 0) SendChangeLine(display, Pager, Pager->pos);
		}
	}
	return 1;
}

CutMotionNotify(Display *display)
{
	XEvent  event;

	while(XPending(display) >= 1){
		XPeekEvent(display, &event);
		if (event.type == MotionNotify){
			XNextEvent(display, &event);
			continue;
		}else{
			break;
		}
	}
}
