#include <string.h>
#include <stdio.h>
#include <jcode.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/keysym.h>
#include <X11/Xlocale.h>

#define	ANK		0
#define	KANJI	1

XIMStyle SelectInputStyle(XIM im);
XIC CreateInputContext(XIM im, XIMStyle style, XFontSet fs, Window p);
void MoveInputPosition(XIC ic, XFontSet fs, char *str, int slen);
void SetGeometry(XIC ic, XEvent e, XIMStyle style, char *attr);

SetJapan(Display *display, Window window, XIC *ic, XIMStyle *style, XFontSet fs)
{
	XIM					im;
	long				mask;
	XWindowAttributes	attr;

    XGetWindowAttributes(display, window, &attr);
    XSelectInput(display, window, ExposureMask|KeyPressMask|StructureNotifyMask|mask|attr.all_event_masks);

    setlocale(LC_ALL, "");
    if (XSupportsLocale() == False) return 1;
    if (XSetLocaleModifiers("") == NULL){
        /* Cannot set locale modifiers. */
        return 2;
    }
    if ((im = XOpenIM(display, NULL, NULL, NULL)) == NULL){
        /* Cannot open input method. */
        return 3;
    }
    *style = SelectInputStyle(im);
    *ic = CreateInputContext(im, *style, fs, window);
    XGetICValues(*ic, XNFilterEvents, &mask, NULL);
    XSetICValues(*ic, XNFocusWindow, window, NULL);
	return 0;
}

char *getstr(display, win_r, win_w, fs, gc, x, y, SIZE_X, SIZE_Y, ic, style, foreground, background, InputMode)
Display			*display;
Window			win_r;
Window			win_w;
XFontSet		fs;
GC				gc;
int				x;
int				y;
int				SIZE_X;
int				SIZE_Y;
XIMStyle        style;
XIC             ic;
unsigned long	foreground;
unsigned long	background;
int				InputMode;
{
	XEvent          e;
	KeySym          sym;
	Status          st;
	char            *buf;
	unsigned int    blen = 1024, len;
	unsigned int    slen = 0;
	int             i;
	char			str[1024];

	if (InputMode == KANJI){
		SetForeground(ic, foreground);
		SetBackground(ic, background);
		SetWindow(ic, win_r, win_w);
		SetFont(ic, fs);
	}

	buf = (char *)malloc(blen);
	while(1){
		if (InputMode == KANJI) SetPoint(ic, x+(slen)*SIZE_X, y);
		XNextEvent(display, &e);
		if (InputMode == KANJI){
			if (XFilterEvent(&e, None)) continue;
		}
		XFillRectangle(display, win_w, gc, x+SIZE_X*slen, y-SIZE_Y+2, SIZE_X, SIZE_Y);
		switch(e.type){
            case    KeyPress:
						if (InputMode == ANK)	len = XLookupString(&e, buf, blen, &sym, NULL);
                        if (InputMode == KANJI)	len = XmbLookupString(ic, &e, buf, blen, &sym, &st);
                        if (InputMode == KANJI && st == XBufferOverflow){
                            buf = (char *)realloc(buf, len);
							blen = len;
							len = XmbLookupString(ic, &e, buf, blen, &sym, &st);
						}
						if (sym == XK_Return){
							str[slen] = 0;
							return str;
						}
						if (sym == XK_BackSpace){
							if (slen <= 0) continue;
							slen--;
							XClearArea(display, win_w, x+slen*SIZE_X, y-SIZE_Y+2, SIZE_X*2, SIZE_Y, False);
                            XmbDrawString(display, win_w, fs, gc, x, y, str, slen);
							XFillRectangle(display, win_w, gc, x+SIZE_X*slen, y-SIZE_Y+2, SIZE_X, SIZE_Y);
							break;
						}
						if (sym == XK_Escape){
							return NULL;
						}
						if (slen > 37) break;
						if (InputMode == ANK){
							XClearArea(display, win_w, x+SIZE_X*slen, y-SIZE_Y+2, SIZE_X*2, SIZE_Y, False);
							for ( i = 0 ; i < len ; i++ )  str[(slen)++] = buf[i];
                            XmbDrawString(display, win_w, fs, gc, x, y, str, slen);
							XFillRectangle(display, win_w, gc, x+SIZE_X*slen, y-SIZE_Y+2, SIZE_X, SIZE_Y);
							break;
						}
						switch(st){
							case    XLookupBoth:
							case    XLookupChars:
										XClearArea(display, win_w, x+SIZE_X*slen, y-SIZE_Y+2, SIZE_X*2, SIZE_Y, False);
										for ( i = 0 ; i < len ; i++ )  str[(slen)++] = buf[i];
                                        if (style & XIMPreeditPosition)  MoveInputPosition(ic, fs, str, slen);
                                        XmbDrawString(display, win_w, fs, gc, x, y, str, slen);
										XFillRectangle(display, win_w, gc, x+SIZE_X*slen, y-SIZE_Y+2, SIZE_X, SIZE_Y);
                                        break;
                        }
                    	break;
			case    Expose:
						if (e.xexpose.count == 0)  XmbDrawString(display, win_w, fs, gc, x, y, str, slen);
                        break;
		}
	}
}


XIMStyle SelectInputStyle(XIM im)
{
	static XIMStyle	preedit[] = {XIMPreeditPosition, XIMPreeditArea, XIMPreeditNothing, NULL};
	static XIMStyle status[]  = {XIMStatusArea, XIMStatusNothing, NULL};
	XIMStyles		*im_styles;
	int				i, j, k;

	XGetIMValues(im, XNQueryInputStyle, &im_styles, NULL);

	i = 0;
	while (preedit[i] != NULL){
		j = 0;
		while (status[j] != NULL){
			for ( k = 0 ; k < im_styles->count_styles ; k++ ){
				if ((preedit[i] & im_styles->supported_styles[k]) && (status[j] & im_styles->supported_styles[k])){
					return im_styles->supported_styles[k];
				}
			}
			j++;
		}
		i++;
	}
	/* Input Style does not match. */
	return NULL;
}

XIC CreateInputContext(XIM im, XIMStyle style, XFontSet fs, Window p)
{
	XIC		ic;
	XVaNestedList	list;
	XPoint			spot;

	spot.x = 10;
	spot.y = 30;
	list = XVaCreateNestedList(0, XNFontSet, fs, XNSpotLocation, &spot, NULL);
	ic = XCreateIC(im, XNInputStyle, style, XNClientWindow, p, XNPreeditAttributes, list, XNStatusAttributes, list, NULL);

	if (ic == NULL){
		/* Cannot create input context. */
		exit (1);
	}
	XFree(list);
	return ic;
}

void MoveInputPosition(XIC ic, XFontSet fs, char *str, int slen)
{
	XPoint		spot;
	XVaNestedList	list;

	spot.x = 10 + XmbTextEscapement(fs, str, slen);
	spot.y = 30;
	list = XVaCreateNestedList(0, XNSpotLocation, &spot, NULL);
	XSetICValues(ic, XNPreeditAttributes, list, NULL);
	XFree(list);
}

void SetGeometry(XIC ic, XEvent e, XIMStyle style, char *attr)
{
	XRectangle		area, *area2;
	XVaNestedList	list;

	area.width = e.xconfigure.width - 20;
	area.height = 18;
	list = XVaCreateNestedList(0, XNAreaNeeded, &area, NULL);
	XSetICValues(ic, attr, list, NULL);
	XFree(list);

	list = XVaCreateNestedList(0, XNAreaNeeded, &area2, NULL);
	XGetICValues(ic, attr, list, NULL);
	XFree(list);

	area2->x = 10;
	if (style == XIMPreeditArea){
		area2->y = e.xconfigure.height - area2->height * 2 - 4;
	}else if (style == XIMStatusArea){
		area2->y = e.xconfigure.height - area2->height - 2;
	}

	list = XVaCreateNestedList(0, XNArea, area2, NULL);
	XSetICValues(ic, attr, list, NULL);
	XFree(list);
	XFree(area2);
}

SetForeground(XIC ic, unsigned long col)
{
	XVaNestedList	list;
	list = XVaCreateNestedList(0, XNForeground, col, NULL);
	XSetICValues(ic, XNPreeditAttributes, list, XNStatusAttributes, list, NULL);
	XFree(list);
}

SetBackground(XIC ic, unsigned long col)
{
	XVaNestedList	list;
	list = XVaCreateNestedList(0, XNBackground, col, NULL);
	XSetICValues(ic, XNPreeditAttributes, list, XNStatusAttributes, list, NULL);
	XFree(list);
}

SetPoint(XIC ic, int x, int y)
{
	XVaNestedList	list;
	XPoint			spot;

	spot.x = x;
	spot.y = y;
	list = XVaCreateNestedList(0, XNSpotLocation, &spot, NULL);
	XSetICValues(ic, XNPreeditAttributes, list, XNStatusAttributes, list, NULL);
	XFree(list);
}

SetWindow(XIC ic, Window win_r, Window win_w)
{
	XSetICValues(ic, XNClientWindow, win_w, XNFocusWindow, win_r, NULL);
}

SetFont(XIC ic, XFontSet fs)
{
	XVaNestedList	list;

	list = XVaCreateNestedList(0, XNFontSet, fs, NULL);
	XSetICValues(ic, XNPreeditAttributes, list, XNStatusAttributes, list, NULL);
	XFree(list);
}
